@@ -58,18 +58,15 @@ struct local_datapath {
     size_t n_allocated_peer_ports;
 
     struct shash external_ports;
+    struct shash multichassis_ports;
 };
 
 struct local_datapath *local_datapath_alloc(
     const struct sbrec_datapath_binding *);
 struct local_datapath *get_local_datapath(const struct hmap *,
                                           uint32_t tunnel_key);
-struct local_datapath *get_local_datapath_no_hash(
-    const struct hmap *local_datapaths,
-    uint32_t tunnel_key);
-
 bool
-need_add_peer_to_local(
+need_add_patch_peer_to_local(
     struct ovsdb_idl_index *sbrec_port_binding_by_name,
     const struct sbrec_port_binding *,
     const struct sbrec_chassis *);
@@ -159,5 +156,10 @@ void add_local_datapath_external_port(struct local_datapath *ld,
                                       char *logical_port, const void *data);
 void remove_local_datapath_external_port(struct local_datapath *ld,
                                          char *logical_port);
+void add_local_datapath_multichassis_port(struct local_datapath *ld,
+                                          char *logical_port,
+                                          const void *data);
+void remove_local_datapath_multichassis_port(struct local_datapath *ld,
+                                             char *logical_port);
 
 #endif /* controller/local_data.h */
