@@ -74,7 +74,6 @@ struct ovn_extend_table;
     OVNACT(CT_LB_MARK,        ovnact_ct_lb)           \
     OVNACT(SELECT,            ovnact_select)          \
     OVNACT(CT_CLEAR,          ovnact_null)            \
-    OVNACT(CT_COMMIT_NAT,     ovnact_ct_nat)          \
     OVNACT(CLONE,             ovnact_nest)            \
     OVNACT(ARP,               ovnact_nest)            \
     OVNACT(ICMP4,             ovnact_nest)            \
@@ -117,6 +116,11 @@ struct ovn_extend_table;
     OVNACT(PUT_FDB,           ovnact_put_fdb)         \
     OVNACT(GET_FDB,           ovnact_get_fdb)         \
     OVNACT(LOOKUP_FDB,        ovnact_lookup_fdb)      \
+    OVNACT(CHECK_IN_PORT_SEC,  ovnact_result)         \
+    OVNACT(CHECK_OUT_PORT_SEC, ovnact_result)         \
+    OVNACT(COMMIT_ECMP_NH,    ovnact_commit_ecmp_nh)  \
+    OVNACT(CHK_ECMP_NH_MAC,   ovnact_result)          \
+    OVNACT(CHK_ECMP_NH,       ovnact_result)          \
 
 /* enum ovnact_type, with a member OVNACT_<ENUM> for each action. */
 enum OVS_PACKED_ENUM ovnact_type {
@@ -256,7 +260,7 @@ struct ovnact_ct_commit_v1 {
     ovs_be128 ct_label, ct_label_mask;
 };
 
-/* OVNACT_CT_DNAT, OVNACT_CT_SNAT, OVNACT_CT_COMMIT_NAT. */
+/* OVNACT_CT_DNAT, OVNACT_CT_SNAT. */
 struct ovnact_ct_nat {
     struct ovnact ovnact;
     int family;
@@ -271,17 +275,9 @@ struct ovnact_ct_nat {
        uint16_t port_hi;
     } port_range;
 
-    bool commit;                /* Explicit commit action. */
-
     uint8_t ltable;             /* Logical table ID of next table. */
 };
 
-enum ovnact_ct_lb_flag {
-    OVNACT_CT_LB_FLAG_NONE,
-    OVNACT_CT_LB_FLAG_SKIP_SNAT,
-    OVNACT_CT_LB_FLAG_FORCE_SNAT,
-};
-
 struct ovnact_ct_lb_dst {
     int family;
     union {
@@ -298,7 +294,6 @@ struct ovnact_ct_lb {
     size_t n_dsts;
     uint8_t ltable;             /* Logical table ID of next table. */
     char *hash_fields;
-    enum ovnact_ct_lb_flag ct_flag;
 };
 
 struct ovnact_select_dst {
@@ -461,6 +456,13 @@ struct ovnact_lookup_fdb {
     struct expr_field dst;     /* 1-bit destination field. */
 };
 
+/* OVNACT_COMMIT_ECMP_NH. */
+struct ovnact_commit_ecmp_nh {
+    struct ovnact ovnact;
+    bool ipv6;
+    uint8_t proto;
+};
+
 /* Internal use by the helpers below. */
 void ovnact_init(struct ovnact *, enum ovnact_type, size_t len);
 void *ovnact_put(struct ofpbuf *, enum ovnact_type, size_t len);
@@ -691,6 +693,9 @@ enum action_opcode {
     /* put_fdb(inport, eth.src).
      */
     ACTION_OPCODE_PUT_FDB,
+
+    /* activation_strategy_rarp() */
+    ACTION_OPCODE_ACTIVATION_STRATEGY_RARP,
 };
 
 /* Header. */
@@ -816,6 +821,10 @@ struct ovnact_encode_params {
                          * 'get_fdb' to resubmit. */
     uint8_t fdb_lookup_ptable; /* OpenFlow table for
                                 * 'lookup_fdb' to resubmit. */
+    uint8_t in_port_sec_ptable; /* OpenFlow table for
+                                * 'check_in_port_sec' to resubmit. */
+    uint8_t out_port_sec_ptable; /* OpenFlow table for
+                                * 'check_out_port_sec' to resubmit. */
     uint32_t ctrl_meter_id;     /* Meter to be used if the resulting flow
                                    sends packets to controller. */
     uint32_t common_nat_ct_zone; /* When performing NAT in a common CT zone,
@@ -830,4 +839,7 @@ void ovnacts_free(struct ovnact[], size_t ovnacts_len);
 char *ovnact_op_to_string(uint32_t);
 int encode_ra_dnssl_opt(char *data, char *buf, int buf_len);
 
+void encode_controller_op(enum action_opcode opcode, uint32_t meter_id,
+                          struct ofpbuf *ofpacts);
+
 #endif /* ovn/actions.h */
